import { RouterController, RouterHandlerType } from '../types/TRouter';
import RouterState, { ItemAndPathType } from './RouterState';
export declare enum EVENTS {
    ROUTE_PROCESSED = "ROUTE_PROCESSED",
    ALL_ROUTES_PROCESSED = "ALL_ROUTES_PROCESSED"
}
export type BaseRouterUseType<T> = RouterController<T>[] | RouterController<T>;
export type ControllerLookupResponseType<T> = Promise<RouterHandlerType<T | void>>;
export type EventListenerCallbackType<T> = (route?: RouterController<T>) => ItemAndPathType<T>;
export type EventListenerType<T> = (event: EVENTS, route?: RouterController<T>) => void;
export type HandlerByEventMapType<T> = {
    [event in EVENTS]?: EventListenerCallbackType<T>;
};
export default abstract class BaseRouter<T> extends RouterState<T> {
    static separator: string;
    protected __allRoutesLoaded: boolean;
    private __isExposed;
    private __totalOfRoutes;
    private __routesProccesed;
    private __eventListener;
    constructor();
    protected addEventListener(listenerHandler: EventListenerType<T>): void;
    private emit;
    private emitPartial;
    /**
     * Function that receives a method and path to find a controller that match with
     * @param method method as string
     * @param path url as string
     * @returns
     */
    protected lookup(method: string, path: string): Promise<ItemAndPathType<T>>;
    /**
     * This function will try to return the function handler for a given route.
     * Due it's async it receives a map that maps the BaseRouter events with the Router that extends this class.
     *
     * With that, this function returns a promise to let the custom router know when a path was loaded
     * @param eventMapLookupController Map of event controllers that will be triggered once an event occurs.
     * @returns
     */
    protected waitAndLookupController(eventMapLookupController: HandlerByEventMapType<T>): Promise<ItemAndPathType<T>>;
    protected getMapKey(route: Pick<RouterController<T>, 'path' | 'method'>, separator?: string): string;
    private useController;
    useAll(routes: BaseRouterUseType<T>, overrides?: Partial<RouterController<T>>): void;
    /**
     * Overloaded method.
     *
     * If args is type of [string,RouterController<T>] add a single controller for that path. Usually you call it when using the util createController.
     * if args is type of [RouterController<T>[]] add a bulk of paths.
     * @param args All arguments.
     */
    use(...args: [string, BaseRouterUseType<T>] | [BaseRouterUseType<T> | any] | [BaseRouterUseType<T>[] | any]): void;
    delete(path: string): T;
    expose(): void;
    static errorToHttpError(e: any): {
        statusCode: any;
        body: any;
    };
}
